!--------------------------------------------------------------------------------
! Copyright (c) 2017 Peter Grünberg Institut, Forschungszentrum Jülich, Germany
! This file is part of FLEUR and available as free software under the conditions
! of the MIT license as expressed in the LICENSE file in more detail.
!--------------------------------------------------------------------------------

MODULE m_writeOutHeader

CONTAINS

  SUBROUTINE writeOutHeader()
    USE m_juDFT
    USE m_utility
    USE m_constants
    USE m_compile_descr
    IMPLICIT NONE
    CHARACTER(len=9) :: cppflag(11)
    CHARACTER(:), ALLOCATABLE:: infostring
    INTEGER          :: i,j

    WRITE (oUnit,*) 'This output is generated by ',version_const
    WRITE (oUnit,*) '                                      * * '
#if ( defined(CPP_AIX) )
    WRITE (oUnit,*)  '                                    * \\:/ *'
#else
    WRITE (oUnit,*)  '                                    * \:/ *'
#endif
    WRITE (oUnit,*)  '                                    *  |  *'
    WRITE (oUnit,*)  '                                      * *  '
    WRITE (oUnit,*) 

    CALL get_compile_desc_string(infostring)
    write(oUnit,'(a)') infostring
    CALL getComputerArchitectures(cppflag,i) ! First determine the architecture
    IF (i.GT.1) THEN 
       WRITE (oUnit,*) 'You set compiler flags for more than one'
       WRITE (oUnit,*) 'architecture: ', (cppflag(j),j=1,i)
       WRITE (oUnit,*) 'Define only one system architecture! '
       CALL juDFT_error("Define only one system architecture! "&
            &         ,calledby ="dimens")
    ENDIF
    IF (i == 0) THEN
       WRITE (oUnit,*) 'No system architecture specified in Makefile'
       cppflag(1) = 'GEN'
    ENDIF
    !
    !       check for double precision etc.
    !

    CALL getAdditionalCompilationFlags(cppflag,i)
    IF (i.GT.0) THEN
       WRITE (oUnit,*) 'Additional flags are: ', (cppflag(j),j=1,i)
    ENDIF

  END SUBROUTINE writeOutHeader

END MODULE m_writeOutHeader
